% STATE EQUATIONS OF THE INDUCTION MACHINE
% in stationary abc frame;
% per-unit formulation. 
% Starting transients
% =========================================================================================
function IMabc2
 clc,close all,clear
        info= ...
        ['                    INDUCTION MOTOR START                            '
         '                     [Phase variable model]                          ' 
         '                          (abc stationary frame)                     '
         '                          (per-unit formulation)                     '
         ' The purpose of this script file is to show how the state equations  '  
         ' of an induction machine are assembled in the context of a simple    '
         ' mode of operation: namely, online starting.                         ' 
         ' The equations are expressed in the abc stationary frame in which the' 
         ' state variables are the machine stator currents ias, ibs, ics; the  ' 
         ' rotor currents iar, ibr, icr; the speed wm; and the displacement    '
         ' angle theta between stator as_axis and rotor ar_axis.               ' 
         ' The state equations are solved using ODE45.                         '
         ' Speed, torque, and current responses are plotted.                   '];
                                     
         h0=msgbox(info,'Induction motor start','help');
         set(h0,'Units','normalized','Resize','on','Position',[.32 .58 .36 .27]) 
  global Lm 
  
 % ---------------------------  ODE  ---------------------------------             
 xo=[0 0 0 0 0 0 0 0]';                 % initial conditions 
 tf=4.0 ;                               % final time (s)
 tspan=[0 tf] ; 
 waitbar(0,'Computing...');
 options=odeset('RelTol',1e-3,'maxstep',2e-3); 
 [t,x]=ode45(@IMabcstate2,tspan,xo,options) ;   
 
 % --------------------  OUTPUT VARIABLES  --------------------------- 
  spd=x(:,7);        % Speed
  ias=x(:,1); ibs=x(:,2); ics=x(:,3);       % Stator 3-phase currents
  Is=sqrt(ias.^2+(ibs-ics).^2/3)  ;         % Stator current magnitude
  iar=x(:,4);                               % Rotor current
 
  % -------------------------------------------Torque calculation
   gam=2*pi/3;
   lg=length(spd);
 for k=1:lg
   h1=waitbar(k/lg);  
    theta=x(k,8);               % Angle between stator as_axis and rotor ar_axis 
    
    i=x(k,1:6);                 % Current vector
        
    dLsr=Lm*[-sin(theta)     -sin(theta+gam) -sin(theta-gam)
            -sin(theta-gam)   -sin(theta)    -sin(theta+gam)
            -sin(theta+gam)  -sin(theta-gam)     -sin(theta)];
        
    dLdtheta=[zeros(3) dLsr
              dLsr'  zeros(3)];
        
    Te(k)=0.5*i*dLdtheta*i'  ;  % Electromagnetic torque
 end
 close(h0,h1)
   % ------------------------- PLOTS ------------------------------------
        IMabcgraf2
       %     edit IMabcgraf2 to view plot instructions
% ========================================================================================

function dxdt=IMabcstate2(t,x) 
global Lm t1 TLi TLf wo
% --------------------     MACHINE PARAMETERS [pu] --------------------
Vs=1.0   ; % Stator voltage 
ws=1.0;    % Stator frequency
freq0=60 ; % Base frequency (Hz)
wo=2*pi*freq0 ; % Base frequency (rad/s)
Rs=0.01  ; % Stator resistance 
Rr=0.02  ; % Rotor resistance  
Lsl=.10 ;  % Stator leakage inductance
Lrl=.10  ; % Rotor leakage inductance  
Lm=4.50;   % Magnetizing inductance
H=0.60 ;   % Inertia constant (s)
Bm=0.10;   % Frictional coefficient
TLi=0.10;  % Initial load torque 
TLf=1.27;  % Final load torque 
t1=2.0 ;   % Time for load application (s)
Ls=Lsl+Lm ; Lr=Lrl+Lm;
    
% ---------------------- INPUTS --------------------------------------------   
  %  Load time profile :    
       if t<t1, TL=TLi; else TL=TLf; end    
  %  Stator voltage time profile  : 
       gam=2*pi/3;
       vas=Vs*cos(ws*wo*t); vbs=Vs*cos(ws*wo*t-gam); vcs=Vs*cos(ws*wo*t+gam);
       
 % ------------------------ STATE EQUATIONS ---------------------------------
  % State  vector:       x=[ias ibs ics iar ibr icr wm theta]'
       wm=x(7) ;                        % rotor speed
       theta=x(8);                      % rotor to stator phase displacement   
       
    V=[vas vbs vcs 0 0 0]';             % Voltage vector
      
    I=x(1:6) ;                          % Current vector
    
    R=diag([Rs Rs Rs Rr Rr Rr]);        % Resistance matrix   
   
    % Inductance matrices :
    Lss=[Ls   -Lm/2 -Lm/2   
        -Lm/2   Ls  -Lm/2
        -Lm/2 -Lm/2   Ls];  
    
    Lrr=[Lr   -Lm/2 -Lm/2   
        -Lm/2   Lr  -Lm/2
        -Lm/2 -Lm/2   Lr]; 
    
    Lsr=Lm*[cos(theta)     cos(theta+gam) cos(theta-gam)
            cos(theta-gam)   cos(theta)   cos(theta+gam)
            cos(theta+gam) cos(theta-gam)     cos(theta)];
    
    dLsr=Lm*[-sin(theta)     -sin(theta+gam) -sin(theta-gam)
            -sin(theta-gam)   -sin(theta)    -sin(theta+gam)
            -sin(theta+gam)  -sin(theta-gam)     -sin(theta)];
              
       
    L=[Lss  Lsr                     % Inductance matrix
       Lsr' Lrr];    
       
    dLdtheta=[zeros(3) dLsr         % dL/dtheta
               dLsr'  zeros(3)];
                   
        
    Te=0.5*I'*dLdtheta*I ;          % Electromagnetic torque
 
  %%% Matrix formulation of induction machine equations in abc stationary reference frame| 
  %%% Constitutive flux-current relations:
  %%%    F = L*I   or  I = L\F  
  %%% Electrical system equations:
  %%%    V= R*I + dF/wodt = R*I + d(L*I)/dwodt = (R + wm*dL/dtheta)*I + L*dI/wodt  
  %%% Mechanical system equation:
  %%%    Te = 2*H*dwm/dt + TL + Bm*wm
    
   didt=L\(V-(R+wm*dLdtheta)*I)*wo;      % Electrical system equations
   dwmdt=(Te-TL)/(2*H)-Bm*wm/(2*H);      % Mechanical system equation
   
   dxdt=[didt;dwmdt;wm*wo];              % State equations obtained by concatenation
% ===================================================================================
