% Synchronous machine
% Phase variable model... Cylindrical rotor
% per unit
 clc,close all,clear
   info= ...
        ['                  SYNCHRONOUS MACHINE TRANSIENTS                 '
         '                             CYLINDRICAL ROTOR                   '
         '                             [Phase variable model]              '
         '                                   [per unit]                    '
         ' A cylindrical rotor synchronous generator is running at a light ' 
         ' loading when suddenly a step reactive load is requested followed' 
         ' by an active load command in the form of either a step or a ramp'
         ' (other types of inputs can be programmed).                      '
         ' The dynamic equations are expressed in canonical first-order    '  
         ' matrix form in the stationary phase variable model in which the '
         ' state variables are the stator currents ias, ibs, ics; the rotor'
         ' field current iff; the d-axis and q-axis damper currents idk and' 
         ' iqk; the angular speed wm; and the displacement angle theta     '
         ' between stator as-axis and rotor direct axis.                   '
         ' The torque, speed,and current responses are plotted including   '
         ' the torque angle  and power factor.                             '];
                                         
       h0=msgbox(info,'Synchronous machine transients','help');
       set(h0,'Units','normalized','Resize','on','Position',[.32 .58 .36 .32]) 
       
 % --------------------     MACHINE PARAMETERS [pu] -----------------
Ra=0.018    ; %Stator resistance 
Ls=1.15     ; %Synchronous inductance
Lal=0.15    ; %Armature leakage inductance
Rf=0.015    ; %Field resistance 
Lf=1.2      ; %Field inductance
Rkd=0.2     ; %D-axis damper resistance
Rkq=0.2     ; %Q-axis damper resistance
Lkd=1.32    ; %D-axis damper inductance tle('Alternator transients','color','r');xlabel('Time [s]'),ylabel('Stator currents [pu]'),grid
Lkq=1.26    ; %Q-axis damper inductance
freq0=60    ; %Frequency [Hz]
wo=2*pi*freq0 ; %Base frequency (rad/s)
H=1.0       ; %Inertia constant [s]
TL1=-0.02   ; %Initial load torque applied a t=0
TL2=-1.0    ; %Final load torque applied at t=t2
Bm=1e-6     ; %Frictional coefficient
Vm=1.0      ; %Stator voltage
vf1=0.016   ; %Initial field dc voltage
vf2=0.03    ; %Final field dc voltage
t1=3.0      ; %Time of load application [s]
t2=1.0      ; %Time of reactive load application [s]
Bm=1e-6     ; %Frictional coefficient
Vm=1.0      ; %Stator voltage
gam=2*pi/3  ;  
ifo=vf1/Rf;
     
%  ---------------------------  IC  -------------------------------------------
                 %  xo=[ias   ibs   ics  iff ikd ikq wm theta]'

 xo=[-0.0133    0.0414   -0.0281    ifo   0.0000    0.0000    1.0000   4.7323]';       % IC

%  These initial conditions are derived from the last set of state variables xo=x(lg,1:8)
%  obtained from an initial run where the inputs TL=TL1 and vf=vf1 remain constant and the            
%  initial conditions are tentatively set to  xo=[0 0 0 ifo 0 0 1 -pi/2]'   

 waitbar(0,'Computing...');    
 
%  ---------------------------  ODE  -------------------------------------------
 tf=10;                          % final time
 tspan=[0 tf] ; 
 options=odeset('RelTol',1e-3,'maxstep',1e-3); 
 [t,x]=ode45(@SMabcf7,tspan,xo,options,Ra,Rf,Rkd,Rkq,Ls,Lal,Lf,Lkd,Lkq,Vm,vf1,vf2,wo,H,Bm,TL1,TL2,tf,t1,t2);
 
   % --------------------  OUTPUT VARIABLES ----------------------------------
 spd=x(:,7);  % speed
 ias=x(:,1)  ;  ibs=x(:,2) ; ics=x(:,3);        % Stator 3-phase instantaneous currents 
 Is=sqrt(ias.^2+(ibs-ics).^2/3)  ;              % Stator current magnitude
 iff=x(:,4)  ;  ikd=x(:,5) ; ikq=x(:,6);        % Rotor currents 
 ang=180/pi*(wo*t-x(:,8));               
 angle=mod(ang,360)-90;                         % Torque angle delta [o]
  lg=length(spd); 
  A=cos(wo*t);
  er=1e-4;
  indx=find(A>=1-er & A<=1+er);
  pf=-ias(indx)./Is(indx);                      % Power factor
  dl=length(pf)-1;
  T=0:tf/dl:tf;

 for k=1:lg
    h1=waitbar(k/lg); 
    theta=x(k,8);                               % Angle between stator and rotor  
    i=x(k,1:6);                                 % Current vector
    
   Lfkd=1.0; Laf=1.0; Lakd=1.0; Lakq=1.0;   
    
    dLsr=[-Laf*sin(theta)     -Lakd*sin(theta)      -Lakq*cos(theta)
          -Laf*sin(theta-gam) -Lakd*sin(theta-gam)  -Lakq*cos(theta-gam)
          -Laf*sin(theta+gam) -Lakq*sin(theta+gam)  -Lakq*cos(theta+gam)];
            
    dLdtheta=[zeros(3) dLsr
              dLsr'  zeros(3)];
        
    Te(k)=0.5*i*dLdtheta*i'  ;                   % Electromagnetic torque  
  end
     close(h0,h1)
  
  % --------------------  GRAPHICAL DISPLAYS  ----------------------------------
  figure('Units','Normalized','Position',[.01 .51 .45 .35],'Name','Alternator');
  figure(1)
  plot(t ,spd ,'color',[0.5 0 0]);title('Alternator','color','r');xlabel('Time [s]'),ylabel('Speed [pu]'),grid
  
   figure('Units','Normalized','Position',[.01 .49 .45 .35],'Name','Alternator');
  figure(2)
  plot(angle ,Te ,'color',[0.5 0 0]);title('Alternator','color','r');xlabel('Torque angle [o]'),ylabel('Torque [pu]'),grid
  
  figure('Units','Normalized','Position',[.46 .49 .45 .35],'Name','Alternator');
  figure(3)
  plot(t ,Te ,'-r');title('Alternator transients','color','r'); xlabel('Time [s]'),ylabel('Torque [pu]'),grid
                          
  figure('Units','Normalized','Position',[.01 .08 .45 .32],'Name','Alternator');
  figure(4) 
   plot(t ,ias ,'-r',t ,ibs ,'-g',t ,ics ,'-b',t,Is,'-k');title('Alternator transients','color','r');xlabel('Time [s]'),ylabel('Stator currents [pu]'),grid
   
  figure('Units','Normalized','Position',[.46 .08 .48 .32],'Name','Alternator'); 
    figure(5) 
   plot(t ,iff ,'-g');title('Alternator transients','color','r');xlabel('Time [s]'),ylabel('Field current [pu]'),grid 
   
   figure('Units','Normalized','Position',[.48 .08 .50 .32],'Name','Alternator transients'); 
    figure(6) 
   plot(angle ,spd ,'color',[.5 .3 .4]);title('Alternator','color','r');xlabel('Torque angle [o]'),ylabel('Speed [pu]'),grid 
   
   figure('Units','Normalized','Position',[.48 .02 .50 .32],'Name','Alternator');
   figure(7)
   plot(T',pf,'-m');title('Alternator','color','r');xlabel('Time [s]'),ylabel('Power factor'),grid
   
      h0=figure('Units','Normalized','menubar','none','numbertitle','off','Position',[.88 .50 .05 .1]); 
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','clc,close all,return', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.21 0.1 0.6 0.22], ...
	      'String','CLOSE', ...
	      'TooltipString','Close'); 
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','smabcrun7', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.21 0.31 0.6 .22], ...
	      'String','RERUN', ...
	      'TooltipString','Run');
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','edit smabcf7', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.21 0.55 0.6 .22], ...
	      'String','STATE', ...
	      'TooltipString','State');
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','edit smabcrun7', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.21 0.77 .6 0.22], ...
	      'String','PARAM', ...
	      'TooltipString','Parameters');
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 