% Synchronous machine transients
% Phase variable model... salient pole rotor
% per unit
 clc,close all,clear
   info= ...
        ['                 SYNCHRONOUS MACHINE STABILITY               '
         '                        SALIENT POLE ROTOR                   '
         '                            [Phase variable model]           '
         '                                  [per unit]                 '
         ' A salient pole rotor synchronous generator is running in    '
         ' steady state under load when a three-phase short occurs at  ' 
         ' the terminals which is then cleared.                        '   
         ' The dynamic equations are expressed in canonical first-     '
         ' order matrix form in the stationary phase variable model in '
         ' which the state variables are the stator currents ias, ibs, ' 
         ' ics; the rotor field current iff; the d-axis and q-axis dam-'
         ' per currents idk and iqk; the angular speed wm; and the     '
         ' displacement angle theta between stator as-axis and rotor   '
         ' direct axis.                                                '
         ' Torque, speed,and current responses are plotted. The torque ' 
         ' angle is also shown.                                        '];
         
       h0=msgbox(info,'Synchronous machine transients','help');
       set(h0,'Units','normalized','Resize','on','Position',[.33 .58 .33 .32]) 
       
 % --------------------     MACHINE PARAMETERS [pu] -----------------
Ra=0.018    ; %Stator resistance 
Ld=1.15     ; %Direct-axis inductance
Lq=0.90     ; %Quadrature-axis inductance
Lal=0.15    ; %Armature leakage inductance
Rf =0.015   ; %Field resistance (proper)
Lf=1.2      ; %Field inductance
Rkd=0.2     ; %D-axis damper resistance
Rkq=0.2     ; %Q-axis damper resistance
Lkd=1.32    ; %D-axis damper inductance
Lkq=1.26    ; %Q-axis damper inductance
freq0=60    ; %Frequency [Hz]
wo=2*pi*freq0 ; %Base frequency (rad/s)
H=4.0       ; %Inertia constant [s]
TL=-0.50    ; %Load torque
vf=0.020    ; %Field excitation
Bm=1e-6     ; %Frictional coefficient
Vm=1.0      ; %Stator voltage
to=0.5      ; %Time of short [s]
tc=0.15     ; %Clearing time [s]
Bm=1e-6     ; %Frictional coefficient
Vm=1.0      ; %Stator voltage
gam=2*pi/3  ;              
ifo=vf/Rf   ; 

         waitbar(0,'Computing...');   
%  ---------------------------  IC  -------------------------------------------
                  %  xo=[ias ibs ics iff ikd ikq wm theta]'

  % xo=[0 0 0 ifo 0 0 1 -pi/2]';  % tentative IC's to establish accurate ones at specified loading 
       
    xo=[-0.3314    0.2780    0.0534  ifo 0 0 1.0 5.0571]';               % accurate IC
 %  These initial conditions are derived from the last set of state variables xo=x(lg,1:8)
 %  obtained from the initial run where no fault occurs (SH=1)
                                                  
 %  ---------------------------  ODE  ---------------------------------------------------------    
 tf=6.0;                            % final time
 tspan=[0 tf] ; 
 options=odeset('RelTol',1e-3,'maxstep',1e-3); 
 [t,x]=ode45(@SMabcstabeq5,tspan,xo,options,Ra,Rf,Rkd,Rkq,Ld,Lq,Lal,Lf,Lkd,Lkq,Vm,wo,H,Bm,TL,vf,to,tc);
 
   % --------------------  OUTPUT VARIABLES ----------------------------------
 spd=x(:,7);  % speed
 ias=x(:,1)  ;  ibs=x(:,2) ; ics=x(:,3);       % Stator currents 
 iff=x(:,4)  ;  ikd=x(:,5) ; ikq=x(:,6);       % Rotor currents  
   ang=wo*t-x(:,8);              
   angle=180/pi*mod(ang,2*pi)-90;              % Torque angle [o]
    Lad=Ld-Lal;   Laq=Lq-Lal;
    L2=(Lad-Laq)/2;
    lg=length(spd); 
 for k=1:lg
    h1=waitbar(k/lg); 
    theta=x(k,8);               % Angle between stator as_axis and rotor direct axis 
    
    i=x(k,1:6);                 % Current vector
   Lfkd=1.0; Laf=1.0; Lakd=1.0; Lakq=1.0;   
   
    dLss=-2*L2*[sin(2*theta)       sin(2*theta-gam)  sin(2*theta+gam)  
                  sin(2*theta-gam) sin(2*(theta-gam))    sin(2*theta)
                  sin(2*theta+gam)   sin(2*theta)    sin(2*(theta+gam))];  
         
    
    dLsr=[-Laf*sin(theta)     -Lakd*sin(theta)      -Lakq*cos(theta)
          -Laf*sin(theta-gam) -Lakd*sin(theta-gam)  -Lakq*cos(theta-gam)
          -Laf*sin(theta+gam) -Lakq*sin(theta+gam)  -Lakq*cos(theta+gam)];
            
    dLdtheta=[dLss   dLsr
              dLsr'  zeros(3)];
        
    Te(k)=0.5*i*dLdtheta*i'  ;  % Electromagnetic torque  
end
  close(h1)
 
  % --------------------  GRAPHICAL DISPLAYS  ----------------------------------
  figure('Units','Normalized','Position',[.01 .49 .45 .35],'Name','Alternator');
  figure(1)
  plot(t,angle,'color',[0.5 0 0]);title('Alternator transients','color','r');xlabel('Time [s]'),ylabel('Torque angle [o]'),grid
  
  figure('Units','Normalized','Position',[.46 .49 .45 .35],'Name','Alternator');
  figure(2)
    plot(t,Te,'-m');title('Alternator transients','color','r'); xlabel('Time [s]'),ylabel('Torque [pu]'),grid
                      
  figure('Units','Normalized','Position',[.01 .08 .45 .32],'Name','Alternator');
  figure(3) 
   plot(t,ias,'-r',t,ibs,'-g',t,ics,'-b');title('Alternator transients','color','r');xlabel('Time [s]'),ylabel('Stator currents [pu]'),grid
   
  figure('Units','Normalized','Position',[.46 .08 .48 .32],'Name','Alternator'); 
    figure(4) 
   plot(t,iff,'-g');xlabel('Time [s]'),ylabel('Field current [pu]'),grid 
   
   figure('Units','Normalized','Position',[.48 .08 .50 .32],'Name','Alternator transients'); 
    figure(5) 
   plot(angle,spd,'color',[.5 .3 .4]);title('Alternator','color','r');xlabel('Torque angle [o]'),ylabel('Speed [pu]'),grid  
   
      h0=figure('Units','Normalized','menubar','none','numbertitle','off','Position',[.88 .50 .05 .1]); 
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','clc,close all,return', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.21 0.1 0.6 0.22], ...
	      'String','CLOSE', ...
	      'TooltipString','Close'); 
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','smabcstab5', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.21 0.31 0.6 .22], ...
	      'String','RERUN', ...
	      'TooltipString','Run');
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','edit smabcstabeq5', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.21 0.55 0.6 .22], ...
	      'String','STATE', ...
	      'TooltipString','State');
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','edit smabcstab5', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.21 0.77 .6 0.22], ...
	      'String','PARAM', ...
	      'TooltipString','Parameters');
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
