% Short circuit of alternator
% Phase variable model... Cylindrical rotor
% per unit
 clc,clear,close all
   info= ...
        ['                     SYNCHRONOUS MACHINE                  '
         '                       SYNCHRONIZATION                    '
         '                         [Phase variable model]           '
         '                                [per unit]                '
         ' An alternator is running  under open circuit conditions  ' 
         ' when connected to a three-phase supply followed by the   '
         ' application of a step load.                              '
         ' The dynamic equations are expressed in canonical first-  ' 
         ' order matrix form in the stationary phase variable model ' 
         ' in which the state variables are the stator currents ias,' 
         ' ibs, ics; the rotor field current iff; the d-axis and q- '
         ' axis damper currents idk and iqk; the angular speed wm;  '
         ' and the displacement angle theta between stator as_axis  '
         ' and rotor direct axis.                                   '
         ' Torque and current responses are plotted.                ' ];
     
       h0=msgbox(info,'Synchronous motor start','help');
       set(h0,'Units','normalized','Resize','on','Position',[.34 .58 .32 .3]) 
       
 % --------------------     MACHINE PARAMETERS [pu] -----------------
Ra=0.02     ; %Stator resistance 
Ls=1.15     ; %Synchronous inductance
Lal=0.15    ; %Armature leakage inductance
Rf=0.001    ; %Field resistance 
Lf=1.2      ; %Field inductance
Rkd=0.15    ; %D-axis damper resistance
Rkq=0.15    ; %Q-axis damper resistance
Lkd=1.32    ; %D-axis damper inductance
Lkq=1.26    ; %Q-axis damper inductance
freq0=60    ; %Frequency [Hz]
wo=2*pi*freq0 ; %Base frequency (rad/s)
H=1.0       ; %Inertia constant [s]
TLf=-0.8    ; %Load torque
Bm=1e-3     ; %Frictional coefficient
Vm=1.0      ; %Stator voltage
ka=1.2      ; %Voltage phase factor (optimum value=1)
kf=1.0      ; %Voltage amplitude factor (optimum value=1)
gam=2*pi/3  ;              
Lm=Ls-Lal;
waitbar(0,'Computing...');    

%  ---------------------------  ODE  -------------------------------------------
 xo=[0 0 0 kf*Vm/Lm 0 0 1 ka*(-pi/2)]';  % initial conditions for open-circuited alternator
 tf=8.0;                                 % final time
 tspan=[0 tf] ; 
 % options=odeset('RelTol',1e-2,'maxstep',1e-2); 
 [t,x]=ode45(@SMabcsynf1,tspan,xo,[],Ra,Rf,Rkd,Rkq,Ls,Lal,Lf,Lkd,Lkq,Vm,wo,H,Bm,TLf,kf);
 
   % --------------------  OUTPUT VARIABLES ----------------------------------
 spd=x(:,7);  % speed
 ias=x(:,1)  ;  ibs=x(:,2) ; ics=x(:,3);        % Stator currents 
 iff=x(:,4)  ;  ikd=x(:,5) ; ikq=x(:,6) ;       % Rotor currents  
 angle=180/pi*(wo*t-pi/2-x(:,8));               % Torque angle [o]
  
  lg=length(spd); 
  for k=1:lg
    h1=waitbar(k/lg); 
    theta=x(k,8);               % Angle between stator as-axis and rotor direct axis 
    
    i=x(k,1:6);                 % Current vector
   Lfkd=1.0; Laf=1.0; Lakd=1.0; Lakq=1.0;   
    
    dLsr=[-Laf*sin(theta)     -Lakd*sin(theta)      -Lakq*cos(theta)
          -Laf*sin(theta-gam) -Lakd*sin(theta-gam)  -Lakq*cos(theta-gam)
          -Laf*sin(theta+gam) -Lakq*sin(theta+gam)  -Lakq*cos(theta+gam)];
            
    dLdtheta=[zeros(3) dLsr
              dLsr'  zeros(3)];
        
    Te(k)=0.5*i*dLdtheta*i'  ;  % Electromagnetic torque  
 end
  close(h1)
  
  % --------------------  GRAPHICAL DISPLAYS  ----------------------------------
  figure('Units','Normalized','Position',[.46 .49 .47 .34],'Name','Alternator sync');
  figure(1)
  plot(t,Te,'-m');title('Torque vs time','color','r');xlabel('Time [s]'),ylabel('Torque [pu]'),grid
  figure('Units','Normalized','Position',[.01 .08 .45 .32],'Name','Alternator sync');
  figure(2) 
   plot(t,ias,'-r');title('Stator current','color','r'),xlabel('Time [s]'),ylabel('ias [pu]'),grid
   figure('Units','Normalized','Position',[.46 .08 .47 .32],'Name','Alternator sync'); 
  figure(3) 
   plot(t,iff,'-g');title('Field current vs time','color','r');xlabel('Time [s]'),ylabel('Field current [pu]'),grid 
   figure('Units','Normalized','Position',[.01 .50 .45 .32],'Name','Alternator sync'); 
  figure(4) 
   plot(t,angle,'-m');title('Torque angle vs time','color','r');xlabel('Time [s]'),ylabel('Torque angle [o]'),grid 
  figure('Units','Normalized','Position',[.02 .53 .45 .32],'Name','Alternator sync'); 
  figure(5) 
   plot(angle,spd,'-c');title('Speed vs angle','color','r');xlabel('Angle[p]'),ylabel('Speed [pu]'),grid 
  
    h0=figure('Units','Normalized','menubar','none','numbertitle','off','Position',[.88 .50 .05 .1]); 
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','clc,close all,return', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.21 0.1 0.6 0.22], ...
	      'String','CLOSE', ...
	      'TooltipString','Close'); 
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','smabcsync1', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.21 0.31 0.6 .22], ...
	      'String','RERUN', ...
	      'TooltipString','Run');
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','edit smabcsynf1', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.21 0.55 0.6 .22], ...
	      'String','STATE', ...
	      'TooltipString','State');
       uicontrol('Parent',h0, ...
          'Units','normalized', ...
	      'BackgroundColor',[0.5 0.5 0.5], ...
	      'Callback','edit smabcsync1', ...
	      'FontSize',10, ...
	      'FontWeight','bold', ...
          'ForegroundColor',[1 1 1], ...
 	      'Position',[0.21 0.77 .6 0.22], ...
	      'String','PARAM', ...
	      'TooltipString','Parameters');
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
            %---- alert ------------
        Fs=4096; 
        w=1500; tt=[0:1/Fs:.15];
        y=sin(w*tt);
       if max(abs(ias(1:200)))>2
            warndlg(' Stator current overload ','alert')
            sound(y,Fs)  
       end
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
